package com.supervision.config;

import com.supervision.common.domain.R;
import com.supervision.common.enums.ResultStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 统一异常处理器配置
 *
 * @author wb
 * @date 2022/3/10 13:24
 */
@Slf4j
@Configuration
@RestControllerAdvice(annotations = RestController.class, basePackages = {"com.supervision.**.controller"})
public class ExceptionHandlerConfig {

    /**
     * 添加手动校验参数的异常处理
     * @param exception 参数验证异常
     * @return 通用返回值
     */
    @ExceptionHandler(IllegalArgumentException.class)
    public R<?> manualValidationExceptionResponse(IllegalArgumentException exception) {
        log.error("=========手动校验参数异常=========>>>");
        log.error(exception.getMessage(), exception);
        log.error("<<<=========手动校验参数异常=========");
        return R.fail(ResultStatusEnum.ILLEGAL_ARGUMENT,exception.getMessage());
    }


}
