package com.supervision.police.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.supervision.common.domain.R;
import com.supervision.police.dto.CaseEvidenceDetailDTO;
import com.supervision.police.dto.EvidenceIdWrapper;
import com.supervision.police.service.CaseEvidenceService;
import com.supervision.police.dto.CaseEvidenceDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;


/**
 * 案件证据控制器
 */
@Slf4j
@RestController
@Tag(name = "案件证据")
@RequiredArgsConstructor
@RequestMapping("/caseEvidence")
public class CaseEvidenceController {

    private final CaseEvidenceService caseEvidenceService;

    @Operation(summary = "保存案件证据信息")
    @PostMapping("/saveEvidence")
    public R<String> saveEvidence(@RequestBody CaseEvidenceDTO caseEvidenceDTO) {
        String evidenceId = caseEvidenceService.saveEvidence(caseEvidenceDTO);
        return R.ok(evidenceId);
    }

    @Operation(summary = "更新案件证据信息")
    @PostMapping("/updateEvidence")
    public R<String> updateEvidence(@RequestBody CaseEvidenceDTO caseEvidenceDTO) {
        String evidenceId =  caseEvidenceService.updateEvidence(caseEvidenceDTO);
        return R.ok(evidenceId);
    }

    @Operation(summary = "删除案件证据信息")
    @DeleteMapping("/deleteEvidence")
    public R<Boolean> deleteEvidence(@RequestBody EvidenceIdWrapper evidenceIdWrapper) {
        Assert.notEmpty(evidenceIdWrapper.getEvidenceId(), "证据id不能为空");
        boolean success = caseEvidenceService.deleteEvidence(evidenceIdWrapper.getEvidenceId());
        return R.ok(success);
    }


    @GetMapping("/queryEvidenceDetail")
    public R<CaseEvidenceDetailDTO> queryEvidenceDetail(@RequestParam @Parameter(name = "evidenceId",description = "证据id") String evidenceId) {
        CaseEvidenceDetailDTO caseEvidenceDetailDTO = caseEvidenceService.queryEvidenceDetail(evidenceId);
        return R.ok(caseEvidenceDetailDTO);
    }
    @Operation(summary = "分页查询案件证据信息列表")
    @GetMapping("/pageListEvidence")
    public R<IPage<CaseEvidenceDetailDTO>> pageListEvidence(@RequestParam @Parameter(name = "caseId",description = "案件id") String caseId,
                                                            @RequestParam(defaultValue = "1") @Parameter(name = "pageNum",description = "页码") Integer pageNum,
                                                            @RequestParam(defaultValue = "10") @Parameter(name = "pageSize",description = "每页数量") Integer pageSize) {
        IPage<CaseEvidenceDetailDTO> pageListEvidence = caseEvidenceService.pageListEvidence(caseId, pageNum, pageSize);
        return R.ok(pageListEvidence);
    }
}
