package com.supervision.police.dto;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.supervision.police.domain.CasePerson;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Data
public class ModelCaseDTO {

    @Schema(description = "主键 案件id")
    private String id;

    @Schema(description = "案件名称")
    private String caseName;

    @Schema(description = "案件类型")
    private String caseType;

    @Schema(description = "案件类型名")
    private String caseTypeName;

    @Schema(description = "案件状态")
    private String caseStatus;

    @Schema(description = "案件状态名")
    private String caseStatusName;

    @Schema(description = "作案方式")
    private String crimeMode;

    @Schema(description = "作案方式名")
    private String crimeModeName;

    @Schema(description = "案件概述")
    private String caseDetail;

    @Schema(description = "立案时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date registerTime;

    @Schema(description = "受理时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date acceptTime;

    /**
     * 冗余字段，方便前端取数，后端勿用
      */
    @Schema(description = "行为人id")
    private String lawActorId;

    /**
     * 冗余字段，方便前端取数，后端勿用
     */
    @Schema(description = "行为人姓名")
    private String lawActorName;

    @Schema(description = "案件行为人")
    private CasePerson lawActor;

    @Schema(description = "案件当事人")
    private List<CasePerson> lawPartyList;

    @Schema(description = "认定结果")
    private String identifyResult;

    @Schema(description = "认定结果名")
    private String identifyResultName;

    @Schema(description = "综合得分")
    private Integer totalScore;

    @Schema(description = "排序序号")
    private Integer indexNum;

    @Schema(description = "更新开始时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateStartTime;

    @Schema(description = "更新结束时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateEndTime;

    @Schema(description = "数据状态")
    private String dataStatus;

    @Schema(description = "案件编号")
    private String caseNo;

    @Schema(description = "创建人ID")
    private String createUserId;

    @Schema(description = "创建时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime createTime;

    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateUserId;

    @Schema(description = "更新时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateTime;

    public void floatLawActorInfo() {
        if (Objects.nonNull(lawActor)){
            this.lawActorName = lawActor.getName();
            this.lawActorId = lawActor.getId();
        }
    }

}
