package com.supervision.police.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.domain.R;
import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.service.Neo4jService;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecordSplit;
import com.supervision.police.domain.NotePrompt;
import com.supervision.police.domain.TripleInfo;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.mapper.NotePromptMapper;
import com.supervision.police.mapper.TripleInfoMapper;
import com.supervision.police.service.ModelRecordTypeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class ModelRecordTypeServiceImpl extends ServiceImpl<ModelRecordTypeMapper, ModelRecordType> implements ModelRecordTypeService {

    private final ModelRecordTypeMapper modelRecordTypeMapper;

    private final NoteRecordSplitMapper noteRecordSplitMapper;

    private final NotePromptMapper notePromptMapper;

    private final TripleInfoMapper tripleInfoMapper;

    private final Neo4jService neo4jService;

    private final OllamaChatClient chatClient;

    @Override
    public List<ModelRecordType> queryType(String name, Integer page, Integer size) {
//        IPage<ModelRecordType> iPage = new Page<>(page, size);
//        iPage  = modelRecordTypeMapper.selectByName(iPage, name);
//        return R.ok(IPages.buildDataMap(iPage));
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(name);

        for (ModelRecordType modelRecordType : list) {
            //笔录内容
            List<NoteRecordSplit> noteRecords = noteRecordSplitMapper.selectByRecordType(modelRecordType.getRecordType());
            modelRecordType.setRecords(noteRecords);
            //提示词
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(modelRecordType.getId());
            modelRecordType.setPrompts(prompts);
        }
        return list;
//        return R.ok(IPages.buildDataMap(iPage));
    }

    @Override
    public ModelRecordType queryByName(String content) {
        Wrapper<ModelRecordType> wrapper = new QueryWrapper<ModelRecordType>().eq("record_type", content);
        return modelRecordTypeMapper.selectOne(wrapper);
    }

    @Override
    public R<?> saveType(ModelRecordType type) {
        String id = type.getId();
        int i = 0;
        if (StringUtils.isEmpty(id)) {
            //新增
            i = modelRecordTypeMapper.insert(type);
        } else {
            i = modelRecordTypeMapper.updateById(type);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> addOrUpdPrompt(NotePrompt prompt) {
        int i = 0;
        if (StringUtils.isEmpty(prompt.getId())) {
            i = notePromptMapper.insert(prompt);
        } else {
            i = notePromptMapper.updateById(prompt);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> delPrompt(NotePrompt prompt) {
        String id = prompt.getId();
        int i = notePromptMapper.deleteById(id);
        if (i > 0) {
            return R.ok("删除成功");
        } else {
            return R.fail("删除失败");
        }
    }


    @Override
    public List<TripleInfo> getThreeInfo(String caseId, String name, String recordId) {
        // TODO 这里应该改成异步的形式,通过异步的形式来进行提取三元组信息,不能每次点击就跑一遍
        return extractTripleInfo(caseId, name, recordId);
    }

    @Override
    public void testExtractThreeInfo() {
        String prompt = """
                请从
                ---
                办案警官问：你是如何想到这个方法骗钱的? 裴金禄回答：因为我是在中卫市沙坡头区中兰铁路上做监理人员，我这样说的话别人会更容易相信。
                ---
                这段话中按(主体类型:行为人姓名;关系:伪造;客体类型:合同名称)提取特定的三元组信息。
                如果未提取到或关系不匹配或客体类型不匹配,则返回空的json即可。
                如果可以提取到三元组关系,则返回格式为{"主体":"主体名称","关系":"关系名称","客体":"客体名称"}。
                例如‘办案警官问:描述一下事情的经过。行为人小明答:我做了一份假的租车合同,然后骗小刚签了这个合同’，该示例存在给定的三元组关系.则结果为:{"主体":"小明","关系":"伪造","客体":"租车合同"}。
                例如‘办案警官问:办案警官问：讲一下那个软件的名称？小明回答：现在叫畅聊。‘该示例不存在给定的三元组关系,则结果为:{}
                请保证提取准确且完整，主体和客体要具体明确，不能是宽泛的概念。提取的客体要必须属于客体类型!

                以只返回JSON数据
                """;
        // 抽取的客体名称一定属于给定的客体类型分类,且抽取的关系完全等于给定的关系。
        Prompt ask = new Prompt(new UserMessage(prompt));
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        log.info("开始分析:");
        ChatResponse call = chatClient.call(ask);
        stopWatch.stop();
        log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
        String content = call.getResult().getOutput().getContent();
        log.info("分析的结果是:{}", content);

         String oldPrompt = """
                 请分析以下内容中所有规定的三元组信息并补充完整，
                 要求：1.返回式：
                 {result：
                    [
                        {startNodeType:'LawActor',entity:'',endNodeType:'FictionalOrgan',property:'',value:'冒充' },
                        {startNodeType:'LawActor',entity:'',endNodeType:'Seal',property:'',value:'伪造'},
                        {startNodeType:'LawActor',entity:'',endNodeType:'BusinessLicense',property:'',value:'伪造'}
                    ]
                 }。
                 2.必须考虑上下文语境分析。
                 3.对话中不包含此类型三元组则返回{ "result": [] }。
                 例子1：办案警官问：你为了骗取更多的钱都做了哪些准备？裴金禄回答：我刚开始我就是自己想了一些关于骗钱的点子，后面为了更不容易让别人识破我为了更佳逼真，我就从网上随便搜了一家租赁公司，我就搜到了兰州胜利机械租赁有限公司，被讯问人：裴金禄我又想到了我管理的中铁北京局和中铁电气化局施工公司。我先是通过百度搜索了“办证”之后就在网页上面弹出了一个页面上面有一个QQ号，我就加上了。加上之后我就将我的要求给他说了，要求他给我刻两个假的公章，一个是兰州胜利机械租赁有限公司合同专用章，另一个是中铁北京局集团有限公司合同专用章。我还要求他给我伪造了一张兰州胜利机械租赁有限公司的营业执照。
                 回答：{"result":[{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"FictionalOrgan","property":"兰州胜利机械租赁有限公司","value":"冒充"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"兰州胜利机械租赁有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"中铁北京局集团有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"BusinessLicense","property":"兰州胜利机械租赁有限公司营业执照","value":"伪造"}]}。
                 例子2:办案警官问：你为什么要在中卫市沙坡头区签订这些合同？裴金禄回答：因为我在这边上班，我在中卫市沙坡头区签订合同更能有说服力，这样才能更好地骗到钱。回答：{ "result": [] }。
                 """;
    }

    private List<TripleInfo> extractTripleInfo(String caseId, String name, String recordId) {
        // 首先获取所有切分后的笔录
        List<NoteRecordSplit> recordSplitList = noteRecordSplitMapper.selectRecord(caseId, name, recordId);
        List<TripleInfo> tripleInfos = new ArrayList<>();
        // 对切分后的笔录进行遍历
        for (NoteRecordSplit record : recordSplitList) {
            // 根据笔录类型找到所有的提取三元组的提示词
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(record.getRecordTypeId());
            // 遍历提示词进行提取
            for (NotePrompt prompt : prompts) {
                if (StringUtils.isEmpty(prompt.getPrompt())) {
                    continue;
                }
                try {
                    StopWatch stopWatch = new StopWatch();
                    // 分析三元组
                    Prompt ask = new Prompt(new UserMessage(prompt.getPrompt() + record.getQuestion() + record.getAnswer()));
                    stopWatch.start();
                    log.info("开始分析:");
                    ChatResponse call = chatClient.call(ask);
                    stopWatch.stop();
                    log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
                    String content = call.getResult().getOutput().getContent();
                    log.info("分析的结果是:{}", content);
                    // 获取从提示词中提取到的三元组信息
                    JSONObject jsonObject = new JSONObject(content);
                    JSONArray threeInfo = jsonObject.getJSONArray("result");
                    for (int i = 0; i < threeInfo.length(); i++) {
                        JSONObject object = threeInfo.getJSONObject(i);
                        String startNodeType = object.getString("startNodeType");
                        String entity = object.getString("entity");
                        String endNodeType = object.getString("endNodeType");
                        String property = object.getString("property");
                        String value = object.getString("value");
                        // 去空,如果存在任何的空值,则忽略
                        if (StrUtil.hasEmpty(startNodeType, entity, endNodeType, property, value)) {
                            continue;
                        }
                        // 将三元组信息进行保存操作
                        TripleInfo tripleInfo = new TripleInfo(entity, property, value, record.getId(), LocalDateTime.now(), startNodeType, endNodeType);
                        tripleInfoMapper.insert(tripleInfo);
                        tripleInfos.add(tripleInfo);
                    }
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return tripleInfos;
    }

    @Override
    public String addNeo4j(List<String> ids) {
        List<TripleInfo> tripleInfos = tripleInfoMapper.selectByIds(ids);
        int i = 0;
        for (TripleInfo tripleInfo : tripleInfos) {
            try {
                //开始节点
                String start = tripleInfo.getStartNode();
                // 首先看是否已经存在了,如果已经存在了,就不添加了
                CaseNode startNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getStartNodeType(), start, "1");
                if (startNode == null) {
                    startNode = new CaseNode(start, tripleInfo.getStartNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(startNode);
                    startNode.setId(save.getId());
                }
                //结束节点
                String end = tripleInfo.getEndNode();
                CaseNode endNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getEndNodeType(), end, "1");
                if (endNode == null) {
                    endNode = new CaseNode(end, tripleInfo.getEndNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(endNode);
                    endNode.setId(save.getId());
                }
                //关系
                Rel rel = new Rel(startNode.getId(), tripleInfo.getRelation(), endNode.getId());
                Rel relation = neo4jService.findRelation(rel);
                if (relation == null) {
                    neo4jService.saveRelation(rel);
                }
                tripleInfo.setAddNeo4j("1");
                int j = tripleInfoMapper.updateById(tripleInfo);
                if (j > 0) {
                    i++;
                }
                // TODO 重复添加的OK了,删除的呢?
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return ("成功插入" + i + "条信息");
    }
}
