package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.utils.IPages;
import com.supervision.common.utils.ListUtils;
import com.supervision.common.utils.StringUtils;
import com.supervision.minio.domain.MinioFile;
import com.supervision.minio.mapper.MinioFileMapper;
import com.supervision.minio.service.MinioService;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.domain.NoteRecordSplit;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.mapper.ModelCaseMapper;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.service.RecordService;
import com.supervision.springaidemo.dto.QARecordNodeDTO;
import com.supervision.springaidemo.util.RecordRegexUtil;
import com.supervision.springaidemo.util.WordReadUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONObject;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class RecordServiceImpl extends ServiceImpl<NoteRecordSplitMapper, NoteRecordSplit> implements RecordService {

    private final NoteRecordSplitMapper noteRecordSplitMapper;

    private final NoteRecordMapper noteRecordMapper;

    private final MinioService minioService;

    private final ModelCaseMapper modelCaseMapper;

    private final MinioFileMapper minioFileMapper;

    private final OllamaChatClient chatClient;

    private final ModelRecordTypeMapper modelRecordTypeMapper;

    private static final String TYPE_TEMPLATE = """
             分类任务: 对话笔记录文本分类。目标: 将给定的对话笔记录分配到预定义的类别中，这些类别包括但不限于：{allTypes}。"
             说明: 提供一段对话笔记录文本，分类器应该识别出对话的主题，并将其归类到上述类别中的一个。
             示例输入: 文本:
             办案警官问：你为了骗取更多的钱都做了哪些准备？。裴金禄回答：我刚开始我就是自己想了一些关于骗钱的点子，后面为了更不容易让别人识破我为了更佳逼真，我就从网上随便搜了一家租赁公司，我就搜到了兰州胜利机械租赁有限公司，我又想到了我管理的中铁北京局和中铁电气化局施工公司。我先是通过百度搜索了“办证”之后就在网页上面弹出了一个页面上面有一个QQ号，我就加上了。加上之后我就将我的要求给他说了，要求他给我刻两个假的公章，一个是兰州胜利机械租赁有限公司合同专用章，另一个是中铁北京局集团有限公司合同专用章。我还要求他给我伪造了一张兰州胜利机械租赁有限公司的营业执照
             预期输出: { type: '诈骗准备' }
             任务要求:
             1. 分类器应当准确地识别对话的主题。
             2. 如果一段对话笔记录包含多个主题，请选择最相关的类别。
             3. 必须考虑上下文语境和专业术语来确定正确的分类。
             对话内容为：{question} {answer}
            """;

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public String uploadRecords(NoteRecord records, List<MultipartFile> fileList) throws IOException {
        //上传文件，获取文件ids
        List<String> fileIds = new ArrayList<>();
        for (MultipartFile file : fileList) {
            String fileId = minioService.uploadFile(file);
            fileIds.add(fileId);
        }
        if (StringUtils.isNotEmpty(records.getFileIds())) {
            records.setFileIds(records.getFileIds() + "," + StringUtils.join(fileIds, ","));
        } else {
            records.setFileIds(StringUtils.join(fileIds, ","));
        }
        //修改案件的行为人、当事人
        if (StringUtils.isNotEmpty(records.getRole()) && !"嫌疑人".equals(records.getRole())) {
            ModelCase modelCase = modelCaseMapper.selectById(records.getCaseId());
            //行为人
            if ("行为人".equals(records.getRole()) && StringUtils.isEmpty(modelCase.getLawActor())) {
                modelCase.setLawActor(records.getName());
                modelCaseMapper.updateById(modelCase);
            } else if ("受害人".equals(records.getRole()) || "证人".equals(records.getRole())) {
                if (StringUtils.isEmpty(modelCase.getLawParty())) {
                    modelCase.setLawParty(records.getName());
                } else {
                    String[] split = modelCase.getLawParty().split(",");
                    if (!Arrays.asList(split).contains(records.getName())) {
                        modelCase.setLawParty(modelCase.getLawParty() + "," + records.getName());
                    }
                }
                modelCaseMapper.updateById(modelCase);
            }
        }
        int i = 0;
        if (StringUtils.isEmpty(records.getId())) {
            i = noteRecordMapper.insert(records);
        } else {
            i = noteRecordMapper.updateById(records);
        }
        //所有对话类型
        List<String> allTypes = modelRecordTypeMapper.getAllType();

        if (i > 0) {
            //拆分笔录
            for (MultipartFile file : fileList) {
                String context = WordReadUtil.readWord(file);
                List<QARecordNodeDTO> qaList = RecordRegexUtil.recordRegex(context, records.getName());
                for (QARecordNodeDTO qa : qaList) {
                    try {
                        NoteRecordSplit noteRecord = new NoteRecordSplit();
                        noteRecord.setCaseId(records.getCaseId());
                        noteRecord.setNoteRecordsId(records.getId());
                        noteRecord.setNoteName(file.getName());
                        noteRecord.setPersonName(records.getName());
                        noteRecord.setQuestion(qa.getQuestion());
                        noteRecord.setAnswer(qa.getAnswer());
                        noteRecord.setCreateTime(LocalDateTime.now());
                        // 开始对笔录进行分类
                        Map<String, String> paramMap = new HashMap<>();
                        paramMap.put("allTypes", CollUtil.join(allTypes, ";"));
                        paramMap.put("question", qa.getQuestion());
                        paramMap.put("answer", qa.getAnswer());
                        Prompt prompt = new Prompt(new UserMessage(StrUtil.format(TYPE_TEMPLATE, paramMap)));
                        StopWatch stopWatch = new StopWatch();
                        stopWatch.start();
                        log.info("开始分析:");
                        ChatResponse call = chatClient.call(prompt);
                        stopWatch.stop();
                        log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
                        String content = call.getResult().getOutput().getContent();
                        log.info("问：{}, 答：{}", qa.getQuestion(), qa.getAnswer());
                        log.info("分析的结果是:{}", content);
                        JSONObject jsonObject = new JSONObject(content);
                        String type = jsonObject.getString("type").trim();
                        System.out.println("问：" + qa.getQuestion() + "答：" + qa.getAnswer());
                        System.out.println("分析的结果是:" + type);
/*                        // todo 写死测试
                        String type = "";
                        if (qa.getQuestion().contains("你为了骗取更多的钱都做了哪些准备")) {
                            type = "诈骗准备";
                        } else {
                            continue;
                        }*/
                        //保存笔录
                        noteRecord.setRecordType(type);
                        noteRecordSplitMapper.insert(noteRecord);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    }
//                    ModelRecordType exist = modelRecordTypeMapper.queryByName(type);
//                    if (exist == null) {
//                        ModelRecordType modelRecordType = new ModelRecordType();
//                        modelRecordType.setRecordType(type);
//                        modelRecordTypeMapper.insert(modelRecordType);
//                    }
                }
            }
            return "保存成功";
        } else {
            return "保存笔录失败";
        }
    }

    @Override
    public Map<String, Object> queryRecords(NoteRecord noteRecords, Integer page, Integer size) {
        LambdaQueryWrapper<NoteRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StringUtils.isNotEmpty(noteRecords.getName()), NoteRecord::getName, noteRecords.getName())
                .eq(NoteRecord::getCaseId, noteRecords.getCaseId())
                .eq(NoteRecord::getDataStatus, "1");
        List<NoteRecord> list = noteRecordMapper.selectList(wrapper);
        // LinkedHashMap 保障顺序
        Map<String, List<NoteRecord>> nameMap = list.stream().filter(item -> StringUtils.isNotBlank(item.getName()))
                .collect(Collectors.groupingBy(NoteRecord::getName, LinkedHashMap::new, Collectors.toList()));
        List<NoteRecord> res = new ArrayList<>();
        for (String name : nameMap.keySet()) {
            NoteRecord noteRecord = new NoteRecord();
            noteRecord.setName(name);
            noteRecord.setChildren(nameMap.get(name));
            res.add(noteRecord);
        }
        List<NoteRecord> pager = ListUtils.Pager(size, page, res);
        for (NoteRecord person : pager) {
            for (NoteRecord noteRecord : person.getChildren()) {
                String fileIds = noteRecord.getFileIds();
                if (StringUtils.isNotEmpty(fileIds)) {
                    noteRecord.setConfessionMaterial(fileIds.split(",").length);
                }
            }
        }
        return IPages.buildDataMap(pager, res.size());
    }

    @Override
    public void delRecords(String id) {
        NoteRecord noteRecord = noteRecordMapper.selectById(id);
        noteRecord.setDataStatus(StringUtils.getUUID());
        noteRecordMapper.updateById(noteRecord);
        String fileIds = noteRecord.getFileIds();
        if (StringUtils.isNotEmpty(fileIds)) {
            //删除文件
            for (String fileId : fileIds.split(",")) {
                MinioFile minioFile = minioFileMapper.selectById(fileId);
                minioFile.setDataStatus(StringUtils.getUUID());
                minioFileMapper.updateById(minioFile);
                minioService.delFile(fileId);
            }
        }
    }

}
